@@ -31,7 +31,7 @@ no warnings;
 our @ISA = qw( Exporter );
 our @EXPORT = qw( interlinear, interstart );
 
-$VERSION = '0.005';
+$VERSION = '0.007';
 $ABSTRACT = 'Interlinear is a program for building metamodels from incomplete, multivariate, discrete dataseries.';
 
 #######################################################################
@@ -59,7 +59,7 @@ my $nearrelaxlimit = 0; #THIS IS THE CEILING FOR THE RELAXATION OF THE RELATIONS
 my $nearconcurrencies = 1; #Requested minimum number of concurrencies for the nearest neighbour method. Minimum value: 1. The more the requested concurrencies, the greatest the precision, the slowest the convergence.
 my $parconcurrencies = 1; #Requested minimum number of concurrencies for the linear interpolations for each parameter of each instance. Minimum value: 1. The more the requested concurrencies, the greatest the precision, the slowest the convergence.
 my $instconcurrencies = 1; #Requested minimum number of concurrencies for the linear interpolations for each instance. Minimum value: 1. The more the requested concurrencies, the greatest the precision, the slowest the convergence.
-my %factlevels = ( pairs => { 1=>3, 2=>3, 3=>3, 4=>3, 5=>3 } ); #The keys are the factor numbers and the values are the number of levels in the data series.
+# my %factlevels = ( pairs => { 1=>3, 2=>3, 3=>3, 4=>3, 5=>3 } ); #The keys are the factor numbers and the values are the number of levels in the data series. OBSOLETE.
 my $minreq_forgrad = [1, 1, 7 ]; #THIS VALUES SPECIFY THE NUMBER OF PARAMETER DIFFERENCES (INTEGER NUMBERS) TELLING HOW WELL-ROOTED IN SIMULATED REALITY (I.E, NEAR FROM IT) A POINT MUST BE TO SELECT IT FOR CALCULATING THE GRADIENTS FOR THE METAMODEL. THEY ARE INTEGERS STARTING FROM 1 OR GREATER.
 # THE FIRST VALUE IS RELATIVE TO THE FACTORS AND TELLS HOW RELAXED A SEARCH IS.
 # THE SECOND VALUE IS RELATIVE TO THE LEVELS. ONE MAY WANT TO KEEP IT TO 1: THE GRADIENTS ARE CALCULATED USING ONLY ADJACENT INSTANCES.
@@ -68,7 +68,6 @@ my $minreq_forgrad = [1, 1, 7 ]; #THIS VALUES SPECIFY THE NUMBER OF PARAMETER DI
 # ONE MAY WANT TO SET TO THE NUMBER OF PARAMETERS.
 # A LARGE NUMBER, WEAK ENTRY BARRIER. NEVER LESS THAN 1.
 
-#my $minreq_forgrad = [ 0, 0 ]; #  THIS VALUES SPECIFY STRENGTHS (LEVELS OF RELIABILITY) INVERSE TO DISTANCES TELLING HOW WELL-ROOTED IN SIMULATED REALITY (I.E, NEAR FROM IT) A POINT MUST BE TO SELECT IT FOR CALCULATING THE GRADIENTS FOR THE METAMODEL. THEY MUST BE VALUES BETWEEN 1 (FOR RECORDED POINT) AND 0 (FOR VERY LOW-RELIABILITY "META"POINTS). THE FIRST VALUE IS RELATIVE TO THE FACTORS. THE SECOND IS RELATIVE TO THE LEVELS. If 0, no entry barrier.
 my $minreq_forinclusion = 0; # THIS VALUE SPECIFIES A STRENGTH VALUE (LEVEL OF RELIABILITY) TELLING HOW WELL-ROOTED IN SIMULATED REALITY A DERIVED POINT (META-POINT) MUST BE FOR INCLUDING IT IN THE SET OF POINTS USED FOR THE METAMODEL.If 0, no entry barrier.
 my $minreq_forcalc = 0; # THIS VALUE SPECIFIES A STRENGTH VALUE (LEVEL OF RELIABILITY) TELLING HOW WELL-ROOTED IN SIMULATED REALITY A DERIVED POINT MUST BE FOR INCLUDING IT IN THE CALCULATIONS FOR CREATING NEW META-POINTS.  A VALUE BETWEEN 1 (JUST SIMULATED POINT) AND 0. If 0, no entry barrier.
 my $minreq_formerge = 0; # THIS VALUE SPECIFIES A STRENGTH VALUE (LEVEL OF RELIABILITY) TELLING HOW WELL-ROOTED IN SIMULATED REALITY A DERIVED POINT MUST BE FOR MERGING IT IN THE CALCULATIONS FOR MERGING IT IN THE METAMODEL. A VALUE BETWEEN 1 (JUST SIMULATED POINT) AND 0 (SIMULATED POINTS AND "META"POINTS WITH THE SAME RIGHT ) MUST BE SPECIFIED. If 0, no entry barrier.
@@ -211,6 +210,10 @@ sub preparearr
 
 
 
+
+
+
+
 sub pythagoras
 {
   my ( $factbag_ref, $levbag_ref, $stepbag_ref ) = @_;
@@ -807,7 +810,7 @@ sub wei
                         {
                           my $soughtval = ( $elt->[2] + ( $elt->[2] * $soughtinc ) );
                           push ( @{ $wand{$key}{vals} }, $soughtval );
-                          push ( @{ $wand{$key}{dists} }, $totdist );
+                          push ( @{ $wand{$key}{dists} }, $totdist );say $tee "FACTLEVELS: " . dump( %factlevels );
                           #say $tee "PUSHING \$soughtval $soughtval INTO \@{ \$wand{\$key}{vals} }: " . dump( @{ $wand{\$key}{vals} } ) . " BECAUSE KEY: $key . " ;
 
                           #my $strength = ( ( $elt->[3] ** $totstrength ) ** ( 1/2 ) );
@@ -916,7 +919,7 @@ sub purelin
                   $count++;
                 }
 
-              }
+              }say $tee "FACTLEVELS: " . dump( %factlevels );
               $index++;
             }
           }
@@ -927,7 +930,7 @@ sub purelin
             {
               if ( ( scalar( @diff1 ) > 0 ) and ( scalar( @diff1 ) == ( 1 + $relaxlimit ) ) )
               { #say $tee "AND \@diff1 IS 1: " . dump( @diff1 );#SAME CONTENT AS THE PREVIOUS WHILE HERE BELOW
-                my @diff2 = diff( \@{ $elt->[1] } , \@{ $el->[1] } ); #say $tee "SO \@diff2: " . dump( @diff2 );
+                my @diff2 = dCHECKINGiff( \@{ $elt->[1] } , \@{ $el->[1] } ); #say $tee "SO \@diff2: " . dump( @diff2 );
 
                 my %h1 = map { split( /-/ , $_ ) } @diff1; #say $tee "THEN \%h1: " . dump( %h1 );
                 my @da1 = keys %h1;
@@ -1305,8 +1308,27 @@ sub interlinear
   close SOURCEFILE;
 
   my ( $aarr_ref, $aarrcopy_ref ) = preparearr( @lines );
+
   my @aarr = @{ $aarr_ref };
 
+
+  sub prepfactlev
+  {
+    my @aarr = @_;
+    my %hsh;
+    foreach my $el ( @aarr )
+    {
+      foreach my $bit ( @{ $el->[1] } )
+      {
+        my ( $head, $tail ) = split( /-/ , $bit );
+        $hsh{pairs}{$head} = $tail;
+      }
+    }
+    return( %hsh );
+  }
+
+  my %factlevels = prepfactlev( @aarr );
+
   my ( $factlev_ref ) = tellstepsize( \%factlevels );
   my %factlev = %{ $factlev_ref }; #say $tee "\%factlev: " . dump( %factlev );
 
@@ -1483,32 +1505,32 @@ __END__
 
 =head1 NAME
 
-Sim::OPT::Interlinear.
-CHECKING
+Sim::OPT::Interlinear
+
 =head1 SYNOPSIS
+
   From Perl (via "re.pl" or in a Perl program):
   interlinear( "/path/to/a/pre-prepared-configfile.pl", "/path/to/a/pre-prepared-sourcefile.csv" );
-
   or from the command line:
   interlinear .
   (note the dot at the end), to use the file as a script and include the location of the source file directly in the configuration file;
-
-  or from the command line:
+  or, again, from the command line:
   interlinear interstart
   to begin with a dialogue question.
 
 =head1 DESCRIPTION
 
 Interlinear is a program for computing the missing values in multivariate datasieries pre-prepared in csv format.
-The program can adopt the following algorithmic strategies and intermix their result: a) propagating distance-weighted gradient-based strategy (by far the best one so far, keeping into account that the behaviour of factors is often not linear and there are curvatures all aroung the design space); b) pure linear interpolation (one may want to use this in some occasions: for example, if there are factorials); c) nearest neighbour. This is a strategy of last resort. One may want to use it to unlock a computation which is based on data which are too sparse to proceed, or when nothing else works.
+The program can adopt the following algorithmic strategies and intermix their result:
+a) a propagating distance-weighted gradient-based strategy (by far the best one so far, keeping into account that the behaviour of factors is often not linear and there are curvatures all aroung the design space);
+b) pure linear interpolation (one may want to use this in some occasions: for example, on factorials);
+c) nearest neighbour (a strategy of last resort. One may want to use it to unlock a computation which is based on data which are too sparse to proceed, or when nothing else works).
 A configuration file should be prepared following the example in the "examples" folder in this distribution.
-If the configuration file is incomplete or missing, the program adopts its own defaults; which adopt the distance-weighted gradient-based strategy.
-The only variables that must mandatorily be specified in a configuration file are $sourcefile and (curently)%factlevels.
-The operations entailed are linear but the results of their interplay  are not.
-The operations can be weighted and the strategies can be intermixed.
-The source cvs file has to be prepared listing in cvs format in each column the parameter (factor, variable) values (levels) and the last column the objective function value, when present.
-The parameter (variable, factor) number is given by the position of the column (i.e. column 4 host parameter 4).
-Here below is shown an example of multivatiate dataseries of 3 parameters assuming 3 levels with missing objecive function entries.
+If the configuration file is incomplete or missing, the program adopts its own defaults, exploiting the distance-weighted gradient-based strategy.
+The only variable that must mandatorily be specified in a configuration file is $sourcefile : the Unix path to the source file containining the dataseries.
+The source file has to be prepared by listing in each column the values (levels) of the parameters (factors, variables), putting in the last column the objective function value, when present.
+The parameter number is given by the position of the column (i.e. column 4 host parameter 4).
+Here below is shown an example of multivatiate dataseries of 3 parameters assuming 3 levels each. having with missing objecive function entries.
 
 1,1,1,1.234
 1,2,3,2,1.500
@@ -1520,6 +1542,7 @@ Here below is shown an example of multivatiate dataseries of 3 parameters assumi
 3,2,3,2,0.670
 3,3,3,3
 
+Note that the parameter listings cannot be incomplete. Just the objective function entries can be.
 The program converts this format into the one liked by Sim::OPTS, which is the following:
 
 1-1_2-1_3-1,9.234
@@ -1532,7 +1555,7 @@ The program converts this format into the one liked by Sim::OPTS, which is the f
 1-3_2-2_3-2,0.670
 1-3_2-3_3-3
 
-After some computations, the program will output a new dataseries, with the missing values filled in.
+After some computations, Interlinear will output a new dataseries, with the missing values filled in.
 
 
 =head2 EXPORTUser menu
